from flask_login import UserMixin
from datetime import datetime
from .. import db

class User(db.Model, UserMixin):
    __tablename__ = 'users'

    id = db.Column(db.Integer, primary_key=True)
    email = db.Column(db.String(120), unique=True, nullable=False)
    password_hash = db.Column(db.String(128), nullable=False)
    role = db.Column(db.String(50), default='user')
    is_verified = db.Column(db.Boolean, default=False)
    excluded_from_analytics = db.Column(db.Boolean, default=False)
    created_at = db.Column(db.DateTime, default=datetime.utcnow)

    # Optional: relationship to submissions
    submissions = db.relationship('Submission', backref='user', lazy=True)


class Submission(db.Model):
    __tablename__ = 'submission'

    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(
        db.Integer,
        db.ForeignKey('users.id', name='fk_submission_user_id'),
        nullable=False
    )
    common_name = db.Column(db.String(120), nullable=False)
    scientific_name = db.Column(db.String(120))
    price = db.Column(db.Float, nullable=False)
    source = db.Column(db.String(120))
    timestamp = db.Column(db.DateTime, default=datetime.utcnow)
    height = db.Column(db.Float)
    width = db.Column(db.Float)
    leaf_count = db.Column(db.Integer)
    potting_mix = db.Column(db.String(255))
    container_size = db.Column(db.String(120))
    health_status = db.Column(db.String(50))
    notes = db.Column(db.Text)
    plant_id = db.Column(db.Integer)
    images = db.relationship('SubmissionImage', backref='submission', lazy=True)


class SubmissionImage(db.Model):
    __tablename__ = 'submission_images'

    id = db.Column(db.Integer, primary_key=True)
    submission_id = db.Column(db.Integer, db.ForeignKey('submission.id'), nullable=False)
    file_path = db.Column(db.String(255), nullable=False)


class PlantCommonName(db.Model):
    __tablename__ = 'plants_common'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(120), unique=True, nullable=False)


class PlantScientificName(db.Model):
    __tablename__ = 'plants_scientific'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(255), unique=True, nullable=False)


class Plant(db.Model):
    __tablename__ = 'plants'
    id = db.Column(db.Integer, primary_key=True)
    common_name_id = db.Column(db.Integer, db.ForeignKey('plants_common.id'))
    scientific_name_id = db.Column(db.Integer, db.ForeignKey('plants_scientific.id'))
    parent_id = db.Column(db.Integer, db.ForeignKey('plants.id'), nullable=True)
    is_dead = db.Column(db.Boolean, default=False)
    date_added = db.Column(db.DateTime, default=datetime.utcnow)
    created_by_user_id = db.Column(db.Integer, db.ForeignKey('users.id'))

    # Relationships
    updates = db.relationship('PlantUpdate', backref='plant', lazy=True)
    lineage = db.relationship('PlantLineage', backref='child', lazy=True,
                              foreign_keys='PlantLineage.child_plant_id')


class PlantLineage(db.Model):
    __tablename__ = 'plant_lineage'
    id = db.Column(db.Integer, primary_key=True)
    parent_plant_id = db.Column(db.Integer, db.ForeignKey('plants.id'), nullable=False)
    child_plant_id = db.Column(db.Integer, db.ForeignKey('plants.id'), nullable=False)


class PlantOwnershipLog(db.Model):
    __tablename__ = 'plant_ownership_log'
    id = db.Column(db.Integer, primary_key=True)
    plant_id = db.Column(db.Integer, db.ForeignKey('plants.id'), nullable=False)
    user_id = db.Column(db.Integer, db.ForeignKey('users.id'), nullable=False)
    date_acquired = db.Column(db.DateTime, default=datetime.utcnow)
    date_relinquished = db.Column(db.DateTime, nullable=True)


class PlantUpdate(db.Model):
    __tablename__ = 'plant_updates'
    id = db.Column(db.Integer, primary_key=True)
    plant_id = db.Column(db.Integer, db.ForeignKey('plants.id'), nullable=False)
    update_type = db.Column(db.String(100))
    description = db.Column(db.Text)
    created_at = db.Column(db.DateTime, default=datetime.utcnow)
    images = db.relationship('UpdateImage', backref='update', lazy=True)


class UpdateImage(db.Model):
    __tablename__ = 'update_images'
    id = db.Column(db.Integer, primary_key=True)
    update_id = db.Column(db.Integer, db.ForeignKey('plant_updates.id'), nullable=False)
    file_path = db.Column(db.String(255), nullable=False)


class ImageHeart(db.Model):
    __tablename__ = 'image_hearts'
    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey('users.id'), nullable=False)
    submission_image_id = db.Column(db.Integer, db.ForeignKey('submission_images.id'), nullable=False)
    created_at = db.Column(db.DateTime, default=datetime.utcnow)


class FeaturedImage(db.Model):
    __tablename__ = 'featured_images'
    id = db.Column(db.Integer, primary_key=True)
    submission_image_id = db.Column(db.Integer, db.ForeignKey('submission_images.id'), nullable=False)
    override_text = db.Column(db.String(255), nullable=True)
    is_featured = db.Column(db.Boolean, default=True)
